unit OzekiSMSServer;

interface

uses
  Windows, Messages, SysUtils, Classes;

const
  //Windows messages
  WM_SMS_REJECTED     = WM_APP+7800;
  WM_SMS_ERROR        = WM_APP+7801;

  //SMS Error codes
  SMSERROR_BUSY               = 100;
  SMSERROR_SENDFAILED         = 101;
  CONNECTIONERROR_LINKDOWN    = 202;
  CONNECTIONERROR_NORESPONSE  = 203;
  CONNECTIONERROR_INVALIDUSER = 204;
  UNKNOWNERROR                = 500;

type
  TNotificationEvent = Procedure of Object ;
  TNotificationEventWithStatus = Procedure(statuscode : integer;statusmessage:string) of Object ;
  TSMSReceivedEvent = Procedure(originator,recepient,messagedata,messagetype,operator:widestring) of Object ;
  TSMSSendEvent = procedure(smsid:string) of object;

  TOzekiSMSServer = class(TComponent)
  private
    { Private declarations }
    fHostName : string;
    fHostPort : integer;
    fUserName : string;
    fPassword : string;
    fConnected: boolean;
    fAutoReconnect : boolean;
    instanceId: integer;
    driverID  : integer;
    handle    : hwnd;

    FServerConnecting : TNotificationEvent;
    FServerConnected : TNotificationEvent;
    FServerCouldNotConnect : TNotificationEvent;
    FServerDisConnected : TNotificationEvent;
    FLastError : TNotificationEventWithStatus;

    FSMSTransmitted : TSMSSendEvent;
    FSMSReceived : TSMSReceivedEvent;
    FSMSUndeliverable : TSMSSendEvent;
    FSMSDelivered : TSMSSendEvent;
    FSMSAccepted : TSMSSendEvent;
    FSMSRejected : TSMSSendEvent;

    procedure connect(Value : boolean);
    procedure wndProc(var Msg: TMessage);
    procedure wmSMSRejected(p : pchar);
    procedure wmSMSError(errorCode:integer;p : pchar);
    procedure handleConnectionError;
    procedure signalError(errorCode : integer;errorMessage:string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    connectionState : string;
    lastErrorCode : integer;
    lastErrorMessage : string;
    Constructor Create(AnOwner:TComponent); Override;
    Destructor Destroy; Override;
    function sendSMS(recepient,messagedata:widestring) : string;
    function sendSMSEx(originator,recepient,messagetype,messagedata,operator:widestring) : string;

  published
    { Published declarations }
    property Active :boolean read fConnected write Connect;
    property ServerIP :string read fHostname write fHostName;
    property ServerPort :integer read fHostport write fHostport;
    property UserName : string read fUserName write fUsername;
    property UserPassword : string read fPassword write fPassword;
    property AutoReconnectIfLinkDown : boolean read fAutoReconnect write fAutoReconnect;

    //Events
    property OnServerConnecting : TNotificationEvent Read FServerConnecting Write FServerConnecting;
    property OnServerConnected : TNotificationEvent Read FServerConnected Write FServerConnected;
    property OnServerCouldNotConnect : TNotificationEvent Read FServerCouldNotConnect Write FServerCouldNotConnect;
    property OnServerDisconnected : TNotificationEvent Read FServerDisConnected Write FServerDisConnected;

    property OnSMSReceived : TSMSReceivedEvent Read FSMSReceived Write FSMSReceived;

    property OnSMSTransmitted : TSMSSendEvent Read FSMSTransmitted Write FSMSTransmitted;
    property OnSMSDelivered : TSMSSendEvent Read FSMSDelivered Write FSMSDelivered;
    property OnSMSUndeliverable : TSMSSendEvent Read FSMSUndeliverable Write FSMSUndeliverable;
    property OnSMSAccepted : TSMSSendEvent Read FSMSAccepted Write FSMSAccepted;
    property OnSMSRejected : TSMSSendEvent Read FSMSRejected Write FSMSRejected;

    property OnError : TNotificationEventWithStatus Read FLastError Write FLastError;
  end;

//**************************
//interface to the dll
//**************************
function sendInfoStart(driverhost:integer;action,inparam:pchar):pchar; stdcall
procedure sendInfoEnd(memToFree:pchar); stdcall
function createTransportDriver(driverNameX : pchar;instanceName: pchar; driverHost:integer; procAddrStart,procAddrEnd: pointer):integer; stdcall  external 'ozclient.dll';
function infoReceivedStart(driverAddr:integer; action,param:pchar) : pchar; stdcall external 'ozclient.dll';
procedure infoReceivedEnd(pVal : pchar); stdcall external 'ozclient.dll';
procedure setLogin(driverAddr:integer;dhostname:pchar; dport : integer; dusername:pchar; dpassword: pchar); stdcall external 'ozclient.dll';

procedure Register;

implementation
uses unEnvelope;

var
instances : array[0..1000] of TOzekiSMSServer;
instancecount : integer;

function strpad(number : string; digits:integer):string;
var numzer,x : integer;
    ret : string;
Begin
  ret := number;
  numzer := digits-length(number);
  for x := 1 to numzer do ret := '0'+ret;
  Result := ret;
End;

function toHexchar(i:integer):char;
begin
if i<10 then Result := chr(ord('0')+i)
        else Result := chr(ord('A')+i-10);
end;

function ozinttohex(i:integer) : string;
var ret : string;
Begin
    ret := '';
    if i=0 then ret := '0';
    while i>0 do begin
      ret := tohexchar(i mod 16)+ret;
      i := i div 16;
    end;
    result := ret;
End;


function fromHexchar(i:char):integer;
var res : integer;
begin
if i<'A' then res:= ord(i)-ord('0')
         else res:= ord(i)-ord('A')+10;
Result := res;
end;


function isUnicode(s:widestring):boolean;
var
   wc : widechar;
   pwc : pointer;
   ret : boolean;
   x : integer;
   l : integer;
   b : word;
Begin
   l := length(s);
   ret := false;
   for x := 1 to l do begin
       wc := s[x];
       pwc := @wc;
       b := word(pwc^);
       if b>255 then ret := true;
   end;
   result := ret
End;

function wide2string(s:widestring):string;
var
   wc : widechar;
   pwc : pointer;
   rets : string;
   x : integer;
   l : integer;
   b : word;
Begin
   l := length(s);
   rets := '';
   for x := 1 to l do begin
       wc := s[x];
       pwc := @wc;
       b := word(pwc^);
       rets := rets+strpad(ozinttohex(b),4);
   end;
   result := rets;
End;

function string2wide(s:string):widestring;
var x,l : integer;
    wc : widechar;
    rets : widestring;

Begin
  x := 1;
  l := length(s);
  while x<l do begin
     wc := widechar(fromHexChar(s[x])*16*16*16+fromHexChar(s[x+1])*16*16+fromHexChar(s[x+2])*16+fromHexChar(s[x+3]));
     rets := rets + wc;
     x := x+4;
  end;
  result := rets;
End;

function  fromUCS2(inp:string):widestring;
Begin
    if (pos('_UCS2_',inp)>0) or (pos('UCS2',inp)>0) then begin
        Result := string2wide(chopfront(inp,6))
    end else begin
        Result := inp;
    end;
End;



function singleline(s:string):string;
var
   x,l : integer;
   ts : widestring;
Begin
   l := length(s);
   ts := '';
   for x := 1 to l do begin
     if (s[x]<>chr(13)) and (s[x]<>chr(10)) then begin
        ts := ts+s[x];
     end;
   end;
   Result := ts;
End;

function sendInfoStart(driverhost:integer;action,inparam:pchar):pchar; stdcall;
   var x,ec : integer;
    found,handled : boolean;
    ret,param : string;
    tle : TList;
    env,env2 : Tenvelope;
    p1 : integer;
Begin
   param := singleline(inparam);
   handled := false;
   with TOzekiSMSServer(instances[driverhost]) do begin
   if action='envelope' then begin
       handled := true;
       env2 := tEnvelope.Create(fUserName);
       env2.fromXml(param);
       if env2.messagetype = 'DELIVERYREPORT' then begin
          if env2.messagedata = '1' then begin
             if assigned(onSMSTransmitted) then onSMSTransmitted(env2.callbackid);
          end else if env2.messagedata = '2' then begin
             if assigned(onSMSUndeliverable) then onSMSUndeliverable(env2.callbackid);
          end else if env2.messagedata = '0' then begin
             if assigned(onSMSDelivered) then onSMSDelivered(env2.callbackid);
          end;
       end else begin
          if assigned(onSMSReceived) then OnSMSReceived(env2.originator,env2.recepient,fromUCS2(env2.messagedata),env2.messagetype,env2.operator);
       end;
       env2.Free;
   end else if action='sendSuccess' then begin
       handled := true;
       p1 := pos(';',param);
       param := chopend(param,length(param)-p1+1);
       if assigned(onSMSAccepted) then OnSMSAccepted(param);
   end else if action='sendFailed' then begin
       handled := true;
       p1 := pos(';',param);
       param := chopend(param,length(param)-p1+1);
       if assigned(onError) then OnError(SMSERROR_SENDFAILED,'Could not send message to the message engine');
       if assigned(onSMSRejected) then OnSMSRejected(param);
   end else if action='getSystemInfo' then begin
       handled := true;
   end else if action='work' then begin
       handled := true;
   end else if action='operator' then begin
       handled := true;
   end else if action='driverCom' then begin
       handled := true;
   end else if action='connectionState' then begin
       handled := true;
       if (param='connecting') then begin
            if assigned(FServerConnecting) then FServerConnecting;
       end else if (param='online') then begin
            if assigned(FServerConnected) then FServerConnected;
       end else if (param='offline') then begin
            if connectionstate='connecting' then begin
                if assigned(FServerCouldNotConnect) then FServerCouldNotConnect;
            end else begin
                if assigned(FServerDisconnected) then FServerDisConnected;
            end;
            connectionState:='offline';
            if not fAutoReconnect then begin
              connect(false);
            end;
       end;
       if param<>connectionstate then begin
          TOzekiSMSServer(instances[driverhost]).connectionstate := param;
       end;
   end else if action='messageState' then begin
       handled := true;
   end else if action='driverEvent' then begin
       handled := true;
   end else if action='connectionLinkDown' then begin
       handled := true;
       signalError(CONNECTIONERROR_LINKDOWN,param);
       handleConnectionError;
   end else if action='connectionInvalidUser' then begin
       handled := true;
       signalError(CONNECTIONERROR_INVALIDUSER,param);
   end else if action='connectionNoresponse' then begin
       handled := true;
       signalError(CONNECTIONERROR_NORESPONSE,param);
   end else if action='unknownError' then begin
       handled := true;
       signalError(UNKNOWNERROR,param);
   end else if action='driverEventDetailed' then begin
       handled := true;
   end;
   if not handled then begin
      ret := 'notHandled';
      //addTolog('Received unknown info '+action+';'+param);
   end;
   end;
   Result := nil;
End;

procedure sendInfoEnd(memToFree:pchar); stdcall;
Begin
   if assigned(memToFree) then Freemem(memToFree);
End;

Constructor TOzekiSMSServer.Create(AnOwner:TComponent);
Begin
    inherited create(AnOwner);
    fHostName  := '127.0.0.1';
    fHostPort  := 9334;
    fConnected := false;
    fUserName  := 'admin';
    fPassword  := 'abc123';
    instanceId := instancecount;
    connectionstate := 'offline';
    fAutoReconnect := false;

    driverID := createTransportDriver('','',instanceCount,@sendInfoStart,@sendInfoEnd);
    instances[instanceCount] := self;
    inc(instanceCount);

   //**********************************************************
   //Register message processor function to windows
   //**********************************************************
   handle       := AllocateHwnd(wndProc);

End;

Destructor TOzekiSMSServer.Destroy; 
Begin
    deallocateHWnd(handle);
    inherited destroy;
End;

procedure TOzekiSMSServer.wmSMSRejected(p : pchar);
Begin
   if assigned(FSMSRejected) then FSMSRejected(p);
   freemem(p);
End;

procedure TOzekiSMSServer.signalError(errorCode:integer;errorMessage:string);
Begin
   lastErrorCode := errorCode;
   lastErrorMessage := errorMessage;
   if assigned(FLastError) then FLastError(errorCode,errorMessage);

End;

procedure TOzekiSMSServer.handleConnectionError;
Begin
   if connectionstate='connecting' then begin
        if assigned(FServerCouldNotConnect) then FServerCouldNotConnect;
   end else if assigned(FServerDisconnected) then FServerDisConnected;
   connectionState:='offline';
   if not fAutoReconnect then connect(false);
End;

procedure TOzekiSMSServer.wmSMSError(errorCode:integer;p : pchar);
Begin
   signalError(errorCode,p);
   freemem(p);
End;

procedure TOzekiSMSServer.wndProc(var Msg: TMessage);
Begin
  case Msg.Msg of
     WM_SMS_REJECTED : wmSMSRejected(pchar(msg.lparam));
     WM_SMS_ERROR : wmSMSError(msg.WParam,pchar(msg.lparam));
  else
     defWindowProc(handle,Msg.Msg,Msg.wParam,msg.lParam);
  end;
End;


procedure TOzekiSMSServer.connect(Value : boolean);
Begin
    if Value then begin
      fConnected := true;
      if connectionState='offline' then begin
         setLogin(driverID,pchar(fHostname),fHostPort,pchar(fUserName),pchar(fPassword));
         infoReceivedStart(driverID,pchar('connect'),nil);
      end;
    end else begin
      fConnected := false;
      infoReceivedStart(driverID,pchar('disconnect'),nil);
    end;
End;


function TOzekiSMSServer.sendSMS(recepient,messagedata:widestring) : string;
var msgtype : string;
Begin
   //Send a message and return the message reference id
   if isUnicode(messagedata) then msgtype := 'SMS:TEXT:UCS2' else msgtype := 'SMS:TEXT';
   Result := sendSMSEx('+36201234567',recepient,msgtype,messagedata,'');
End;

function TOzekiSMSServer.sendSMSEx(originator,recepient,messagetype,
         messagedata,operator:widestring) : string;
var smsid : string;
    env : TEnvelope;
    ret : pchar;
    tret : pchar;
    p1 : pchar;
Begin
   //Send a message and return the message reference id
   env := TEnvelope.create(fUserName);
   env.originator := singleline(originator);
   env.recepient := singleline(recepient);
   env.messagetype := singleline(messagetype);
   if env.messagetype='SMS:TEXT:UCS2' then begin
       env.messagedata := '_UCS2_'+wide2string(messagedata);
   end else begin
       env.messagedata := singleline(messagedata);
   end;
   env.subject := env.messagedata;
   env.operator := singleline(operator);
   ret := infoReceivedStart(driverId,pchar('envelope'),pchar(env.toXml));
   if ret <> 'accepted' then begin
      getMem(p1,length(ret)+1);
      StrPCopy(p1, ret);
      postmessage(handle,WM_SMS_ERROR,SMSERROR_BUSY,integer(p1));

      getMem(p1,length(env.id)+1);
      StrPCopy(p1, env.id);
      postmessage(handle,WM_SMS_REJECTED,0,integer(p1));
   end;
   infoReceivedEnd(ret);
   Result := env.id;
   env.Free;
End;

procedure Register;
begin
  RegisterComponents('Ozeki', [TOzekiSMSServer]);
end;

initialization
instancecount := 0;

end.
 